from .extractors import eztv, yts
import os
from halo import Halo


class Ezflix(object):
    def __init__(self,
                 query,
                 media_type='tv',
                 limit=20,
                 sort_by='seeds',
                 sort_order='desc',
                 quality=None,
                 minimum_rating=None,
                 language='en',
                 page=1,
                 debug=False
                 ):
        self._torrents = []
        self._query = query
        self._media_type = media_type
        self._limit = limit
        self._sort_by = sort_by
        self._sort_order = sort_order
        self._quality = quality
        self._minimum_rating = minimum_rating
        self._language = language
        self._page = page
        self._debug = debug

    def magnet(self, val):
        for magnet in self._torrents:
            if magnet['id'] == int(val):
                return magnet
        return None

    def search_subtitles(self, media_title):
        os.system("subliminal download -l %s '%s'" % (self._language, media_title))
        cur_dir = os.getcwd()
        file_list = os.listdir(cur_dir)
        for f in file_list:
            if media_title in f:
                return f

        return None

    def search(self):
        spinner = Halo(text='Searching...', spinner='dots')
        spinner.start()
        if self._media_type == 'tv':
            self._torrents = eztv(self._query.replace(' ', '-').lower(), page=self._page, limit=self._limit,
                                  quality=self._quality, debug=self._debug)
        elif self._media_type == 'movie':
            self._torrents = yts(query_term=self._query,
                                 limit=self._limit,
                                 sort_by=self._sort_by,
                                 sort_order=self._sort_order,
                                 quality=self._quality,
                                 minimum_rating=self._minimum_rating,
                                 page=self._page,
                                 debug=self._debug
                                 )
        spinner.stop()
        return self._torrents



from .utils import cmd_exists, peerflix
import sys
import colorful
from .argument_parser import Parser
from .ezflix import Ezflix
from prettytable import PrettyTable
from builtins import input
import logging
import time
import subprocess
from .extractors.yts import find_similar

logging.basicConfig()
logger = logging.getLogger(__file__)

if sys.version_info < (2, 7):
    logger.error('You need Python 2.7 or later\n')
    sys.exit(1)

parser = Parser()

if len(sys.argv) == 1:
    parser.error()
    sys.exit(0)

args = parser.parse()

media_player = args.media_player
debug = args.debug

if not cmd_exists('peerflix'):
    sys.exit('This program requires Peerflix. https://github.com/mafintosh/peerflix')

if not cmd_exists('mpv') and args.media_player == 'mpv':
    logger.warning("In ezflix, the default player is mpv. It will fallback to vlc if mpv isn't found. "
                   "You can use the media_player argument to set your media player.")
    media_player = 'vlc'

if not (args.query and args.query.strip()):
    sys.exit(colorful.red("Search query not valid."))

def display_menu():
    print(colorful.bold("Make selection (Select the ID of the media you want to stream):"))
    print("Enter 'quit' to close the program.")
    print("Enter 'next' to see the next page of movies.")
    print("Enter 'prev' to see the previous page of movies.")
    print("Enter 'search' to refine your search.")
    print("Enter 'info' and the id of the torrent to get the movie/tv show overview.")
    print("Enter 'trailer' and the id of the torrent to play the movie trailer.")
    print("Enter 'similar' and the id of the torrent to find similar movies.")
    
def display_table(torrents):
    row = PrettyTable()
    row.field_names = ["Id", "Torrent", "Seeds", "Peers", "Released", "Rating"]
    row.align = 'l'
    t = 0
    for result in torrents:
        if not (result['seeds'] == 0 or result['seeds'] is None) or args.no_seeds is True:
            row.add_row([result['id'], result['title'], result['seeds'], result['peers'], result['release_date'], result['rating']])
            t += 1
    if t > 0 or args.no_seeds:
        print(row)
    else:
        print(colorful.red("No results found."))
        sys.exit(0)

def search(page=1, term=None):
    global ezflix
    ezflix = Ezflix(query=args.query if term is None else term,
                    media_type=args.media_type,
                    limit=int(args.limit),
                    sort_by=args.sort_by,
                    sort_order=args.sort_order,
                    quality=args.quality,
                    minimum_rating=args.minimum_rating,
                    language=args.language,
                    page=page,
                    debug=debug
                    )
    torrents = ezflix.search()
    if torrents is None or len(torrents) == 0:
        sys.exit(colorful.red('No results found.'))
    if args.latest:
        latest = torrents[0]
        file_path = ezflix.search_subtitles(latest['title']) if args.subtitles else None
        print("Playing " + latest['title'])
        time.sleep(2.5)
        peerflix(latest['magnet'], media_player, args.media_type, args.subtitles, args.remove, file_path)
        sys.exit()
    display_table(torrents)
    
def main():
    page = 1
    search(page)
    if not args.no_menu:
        display_menu()
    while True:
        read = input()
        if read == 'quit':
            sys.exit()
        elif read == 'search':
            refined_query = input("Enter search query: ")
            search(page=1, term=refined_query)
            continue
        elif read == 'next':
            page += 1
            search(page)
            continue
        elif read == 'prev':
            if page > 1:
                page -= 1
                search(page)
            continue
        elif 'info' in read:
            sp = read.split()
            if not len(sp) > 1:
                print(colorful.bold(
                    "Incorrect usage. Enter 'info' and the id of the torrent to get the movie/tv show overview."))
                continue
            magnet = ezflix.magnet(sp[1])
            if magnet is None:
                print(colorful.red("Invalid selection."))
                continue
            print(magnet['overview'])
            continue
        elif 'similar' in read:
            sp = read.split()
            if not len(sp) > 1:
                print(colorful.bold(
                    "Incorrect usage. Enter 'similar' and the id of the torrent to find similar movies."))
                continue
            magnet = ezflix.magnet(sp[1])
            if magnet is None:
                print(colorful.red("Invalid selection."))
                continue
            torrents = find_similar(magnet['yts_id'])
            display_table(torrents)
            continue
        elif 'trailer' in read:
            sp = read.split()
            if not len(sp) > 1:
                print(colorful.red(
                    "Incorrect usage. Enter 'trailer' and the id of the torrent to play the movie trailer."))
                continue
            magnet = ezflix.magnet(sp[1])
            if magnet is None:
                print(colorful.red("Invalid selection."))
                continue
            trailer = magnet['trailer']
            if not trailer:
                print(colorful.red("No trailer found."))
                continue
            geometry = "50%"
            subprocess.Popen(
                ['/bin/bash', '-c', "%s https://www.youtube.com/watch?v=%s --geometry=%s" % (media_player, trailer, geometry)])
            continue
        try:
            int_val = int(read)
        except (ValueError, TypeError):
            print(colorful.red('Invalid selection.'))
            continue
        magnet = ezflix.magnet(int_val)
        if not magnet:
            print(colorful.red('Invalid selection.'))
            continue
        print("Playing " + magnet['title'])
        file_path = ezflix.search_subtitles(magnet['title']) if args.subtitles else None
        peerflix(magnet['magnet'], media_player, args.media_type, args.subtitles, args.remove, file_path)
        sys.exit()


if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        sys.exit(1)



import requests
from datetime import datetime
from tmdbv3api import TMDb, TV
from pprint import pprint

def eztv(q, limit, page=1, quality=None, debug=False):
    tmdb = TMDb(debug=True)
    tmdb.api_key = 'e1076b74406e0a7d0efb5318f1b662d0'
    tv = TV()
    sr = tv.search(q)
    if not sr:
        return
    url = 'https://eztv.ag/api/get-torrents'
    details = tv.external_ids(sr[0].id)
    overview = sr[0].overview
    req = requests.get('%s?imdb_id=%s&page=%s&limit=%s' % (url, details['imdb_id'][2:], page, limit))
    if debug:
        print(req.status_code)
    if not req.ok:
        return
    results, count = [], 1
    search_results = req.json()
    if debug:
        pprint(search_results)
    if 'torrents' not in search_results:
        return
    for result in search_results['torrents']:
        obj = {
            'id': count,
            'title': result['title'],
            'magnet': result['magnet_url'],
            'seeds': result['seeds'],
            'peers': result['peers'],
            'overview': overview,
            'rating' : '-',
            'release_date': datetime.fromtimestamp(int(result['date_released_unix'])).strftime('%Y-%m-%d %H:%M:%S')
        }
        if quality is not None:
            if quality.lower() in result['title']:
                results.append(obj)
                count += 1
        else:
            results.append(obj)
            count += 1
    if debug:
        pprint(results)
    return results



# -*- coding: utf-8 -*-

from argparse import ArgumentParser
import os
import sys

HERE = os.path.abspath(os.path.dirname(__file__))

version_ns = {}

with open(os.path.join(HERE, '__version__.py')) as f:
    exec (f.read(), {}, version_ns)


class Parser(object):
    def __init__(self):
        self._parser = ArgumentParser(prog='ezflix')
        self._supported_players = [
            'mpv',
            'vlc',
            'mplayer',
            'smplayer',
            'mpchc',
            'potplayer',
            'webplay',
            'omx',
            'airplay'
        ]

        self._sort_types = [
            'download_count',
            'like_count',
            'date_added',
            'seeds',
            'peers',
            'rating',
            'title',
            'year'
        ]

        self._sort_orders = [
            'asc',
            'desc'
        ]

        self._media_types = [
            'movie',
            'tv'
        ]

        self._qualities = [
            '720p',
            '1080p',
            '3d'
        ]
        self._add_arguments()

    def _add_arguments(self):
        self._parser.add_argument(
            'media_type',
            help="The media type.",
            default='tv',
            nargs='?',
            choices=self._media_types
        )

        self._parser.add_argument(
            'query',
            help="The search query."
        )

        self._parser.add_argument(
            '--limit',
            help="TThe limit of results per page that has been set",
            default='20',
            nargs='?'
        )

        self._parser.add_argument(
            '--minimum_rating',
            help="Used to filter movie by a given minimum IMDb rating",
            nargs='?'
        )

        self._parser.add_argument(
            '--media_player',
            help="The media player.",
            default='mpv',
            nargs='?',
            choices=self._supported_players
        )

        self._parser.add_argument(
            '--latest',
            help="Play the latest TV episode.",
            dest='latest',
            action='store_true'
        )

        self._parser.add_argument(
            '--subtitles',
            help="Load subtitles file.",
            dest='subtitles',
            action='store_true')

        self._parser.add_argument(
            '--sort_by',
            help="Sorts the results by choosen value",
            default='date_added',
            nargs='?',
            choices=self._sort_types
        )

        self._parser.add_argument(
            '--sort_order',
            help="Orders the results by either Ascending or Descending order",
            default='desc',
            nargs='?',
            choices=self._sort_orders
        )

        self._parser.add_argument(
            '--quality',
            help="Used to filter by a given quality.",
            nargs='?',
            choices=self._qualities
        )

        self._parser.add_argument(
            '--genre',
            help='Used to filter by a given genre (See http://www.imdb.com/genre/ for full list)'
        )

        self._parser.add_argument(
            '--remove',
            help='Remove files on exit.',
            default=True,
            dest='remove',
            action='store_true'
        )
        
        self._parser.add_argument(
            '--no_seeds',
            help='Include torrents that have no seeds',
            default=False,
            dest='no_seeds',
            action='store_true'
        )

        self._parser.add_argument(
            '--debug',
            help='Set this flag to print JSON to stdout.',
            default=False,
            dest='debug',
            action='store_true'
        )
        
        self._parser.add_argument(
            '--no-menu',
            help="Set this flag to not show the usage menu on program startup.",
            default=False,
            dest='no_menu',
            action='store_true'
        )
        
        self._parser.add_argument(
            '--language',
            help='Language as IETF code. Set this argument to download subtitles in a given language.',
            default='en'
        )

        self._parser.add_argument(
            '-v',
            '--version',
            action='version',
            version="%(prog)s (" + version_ns['__version__'] + ")"
        )

    def error(self):
        print('''              
                ███████╗███████╗███████╗██╗     ██╗██╗  ██╗
                ██╔════╝╚══███╔╝██╔════╝██║     ██║╚██╗██╔╝
                █████╗    ███╔╝ █████╗  ██║     ██║ ╚███╔╝ 
                ██╔══╝   ███╔╝  ██╔══╝  ██║     ██║ ██╔██╗ 
                ███████╗███████╗██║     ███████╗██║██╔╝ ██╗
                ╚══════╝╚══════╝╚═╝     ╚══════╝╚═╝╚═╝  ╚═╝                                                                                        
        ''')
        print("Ezflix v" + version_ns['__version__'])
        self._parser.print_help()
        sys.exit(0)

    def parse(self):
        return self._parser.parse_args()



import requests
from datetime import datetime
from pprint import pprint

def _build_obj(req, quality, limit, debug):
    if 'movies' not in req['data'] or not req['status'] == 'ok' or not req['data']['movie_count'] > 0:
        return
    arr, count = [], 1
    for r in req['data']['movies']:
        if 'torrents' in r:
            for torrent in r['torrents']:
                title = '%s (%s) (%s)' % (r['title'], r['year'], torrent['quality'])
                obj = {'id': count,
                       'yts_id': r['id'],
                       'title': title,
                       'magnet': torrent['url'],
                       'seeds': torrent['seeds'],
                       'peers': torrent['peers'],
                       'overview': r['synopsis'],
                       'trailer': r['yt_trailer_code'],
                       'rating' : r['rating'],
                       'release_date': datetime.strptime(torrent['date_uploaded'], '%Y-%m-%d %H:%M:%S').date()}
                if quality is not None:
                    if quality == torrent['quality']:
                        arr.append(obj)
                        count += 1
                else:
                    arr.append(obj)
                    count += 1
    if debug:
        pprint(arr)
    return arr[0:limit]

def find_similar(movie_id, quality=None, limit=20):
    req = requests.get('https://yts.ag/api/v2/movie_suggestions.json', params={'movie_id':movie_id})
    req = req.json()
    return _build_obj(req, quality, limit)
   

def yts(query_term, quality=None, limit=20, minimum_rating=4, sort_by='date_added', sort_order='asc', page=1, debug=False):
    params = {
        'query_term': query_term,
        'sort_by': sort_by,
        'sort_order': sort_order,
        'limit': limit,
        'quality': quality,
        'minimum_rating': minimum_rating,
        'page': page

    }
    req = requests.get('https://yts.ag/api/v2/list_movies.json', params=params)
    if debug:
        print(req.status_code)
    req = req.json()
    if debug:
        pprint(req)
    return _build_obj(req, quality, limit, debug)